@@ -1,12 +1,17 @@
-use crate::config::{CompiledPaths, GlobalPassOption, JscTarget, ModuleConfig};
+use crate::config::{CompiledPaths, GlobalPassOption, JsMinifyOptions, JscTarget, ModuleConfig};
+use crate::SwcComments;
 use compat::es2020::export_namespace_from;
 use either::Either;
 use std::cell::RefCell;
 use std::rc::Rc;
 use std::{collections::HashMap, sync::Arc};
 use swc_atoms::JsWord;
+use swc_common::comments::Comments;
+use swc_common::sync::Lrc;
 use swc_common::FileName;
-use swc_common::{chain, comments::Comments, errors::Handler, Mark, SourceMap};
+use swc_common::{chain, errors::Handler, Mark, SourceMap};
+use swc_ecma_ast::Module;
+use swc_ecma_minifier::option::MinifyOptions;
 use swc_ecma_parser::Syntax;
 use swc_ecma_transforms::hygiene::hygiene_with_config;
 use swc_ecma_transforms::modules::util::Scope;
@@ -14,6 +19,8 @@ use swc_ecma_transforms::{
     compat, fixer, helpers, hygiene, modules, optimization::const_modules, pass::Optional,
     proposals::import_assertions, typescript,
 };
+use swc_ecma_transforms_base::ext::MapWithMut;
+use swc_ecma_visit::{as_folder, noop_visit_mut_type, VisitMut};
 
 /// Builder is used to create a high performance `Compiler`.
 pub struct PassBuilder<'a, 'b, P: swc_ecma_visit::Fold> {
@@ -27,6 +34,7 @@ pub struct PassBuilder<'a, 'b, P: swc_ecma_visit::Fold> {
     hygiene: Option<hygiene::Config>,
     fixer: bool,
     inject_helpers: bool,
+    minify: Option<JsMinifyOptions>,
 }
 
 impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
@@ -48,6 +56,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
             env: None,
             fixer: true,
             inject_helpers: true,
+            minify: None,
         }
     }
 
@@ -67,6 +76,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
             global_mark: self.global_mark,
             fixer: self.fixer,
             inject_helpers: self.inject_helpers,
+            minify: self.minify,
         }
     }
 
@@ -75,6 +85,11 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         self
     }
 
+    pub fn minify(mut self, options: Option<JsMinifyOptions>) -> Self {
+        self.minify = options;
+        self
+    }
+
     /// Note: fixer is enabled by default.
     pub fn fixer(mut self, enable: bool) -> Self {
         self.fixer = enable;
@@ -134,7 +149,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
         base: &FileName,
         syntax: Syntax,
         module: Option<ModuleConfig>,
-        comments: Option<&'cmt dyn Comments>,
+        comments: Option<&'cmt SwcComments>,
     ) -> impl 'cmt + swc_ecma_visit::Fold
     where
         P: 'cmt,
@@ -151,7 +166,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
             Either::Left(chain!(
                 import_assertions(),
                 Optional::new(typescript::strip(), syntax.typescript()),
-                swc_ecma_preset_env::preset_env(self.global_mark, comments, env)
+                swc_ecma_preset_env::preset_env(self.global_mark, comments.clone(), env)
             ))
         } else {
             Either::Right(chain!(
@@ -165,7 +180,7 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 Optional::new(
                     compat::es2015(
                         self.global_mark,
-                        comments,
+                        comments.clone(),
                         compat::es2015::Config {
                             for_of: compat::es2015::for_of::Config {
                                 assume_array: self.loose
@@ -187,13 +202,13 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
 
         let module_scope = Rc::new(RefCell::new(Scope::default()));
         chain!(
+            self.pass,
+            compat_pass,
             // module / helper
             Optional::new(
                 modules::import_analysis::import_analyzer(Rc::clone(&module_scope)),
                 need_interop_analysis
             ),
-            self.pass,
-            compat_pass,
             compat::reserved_words::reserved_words(),
             Optional::new(export_namespace_from(), need_interop_analysis),
             Optional::new(helpers::inject_helpers(), self.inject_helpers),
@@ -206,11 +221,55 @@ impl<'a, 'b, P: swc_ecma_visit::Fold> PassBuilder<'a, 'b, P> {
                 module,
                 Rc::clone(&module_scope)
             ),
+            as_folder(MinifierPass {
+                options: self.minify,
+                cm: self.cm.clone(),
+                comments: comments.cloned(),
+                top_level_mark: self.global_mark,
+            }),
             Optional::new(
                 hygiene_with_config(self.hygiene.clone().unwrap_or_default()),
                 self.hygiene.is_some()
             ),
-            Optional::new(fixer(comments), self.fixer),
+            Optional::new(fixer(comments.map(|v| v as &dyn Comments)), self.fixer),
         )
     }
 }
+
+struct MinifierPass {
+    options: Option<JsMinifyOptions>,
+    cm: Lrc<SourceMap>,
+    comments: Option<SwcComments>,
+    top_level_mark: Mark,
+}
+
+impl VisitMut for MinifierPass {
+    noop_visit_mut_type!();
+
+    fn visit_mut_module(&mut self, m: &mut Module) {
+        if let Some(options) = &self.options {
+            let opts = MinifyOptions {
+                compress: options
+                    .compress
+                    .clone()
+                    .into_obj()
+                    .map(|v| v.into_config(self.cm.clone())),
+                mangle: options.mangle.clone().into_obj(),
+                ..Default::default()
+            };
+
+            m.map_with_mut(|m| {
+                swc_ecma_minifier::optimize(
+                    m,
+                    self.cm.clone(),
+                    self.comments.as_ref().map(|v| v as &dyn Comments),
+                    None,
+                    &opts,
+                    &swc_ecma_minifier::option::ExtraOptions {
+                        top_level_mark: self.top_level_mark,
+                    },
+                )
+            })
+        }
+    }
+}
